@@ -1,12 +1,11 @@
 # -*- coding: utf-8 -*-
-__version__ = '1.1'
+__version__ = '1.2'
 __author__  = 'Olivier DUFAILLY'
 __license__ = 'BSD'
 
 import os
 import os.path
 import gzip
-from bisect import bisect_left
 
 # Usual and common return codes
 C_CANCEL  = 0x100
@@ -17,16 +16,11 @@ C_FILE_NOT_FOUND = -4
 C_UNKNOWN        = -5
 C_ERROR   = -1000 # Should be a very wrong case
 
-'''
-Return the file extension in lower char
-'''
 def GetLowExt(fn):
+  ''' Return the file extension in lower char '''
   lfn = fn.lower()
   p = lfn.rfind('.')
-  if p > 0:
-    return lfn[p+1:]  
-
-  return ''
+  return lfn[p+1:] if p>0 else ''
 
 def cmp(a, b):
   return (a > b) - (a < b) 
@@ -56,36 +50,18 @@ class switch(object):
 #
 # Binary Search in an ordered table (ascending order)
 #
-def Arrays_binarySearch(t, key):
-  ''' Locate the leftmost value exactly equal to x '''
-  i = bisect_left(t, key)
-  if i != len(t) and t[i] == key:
-    return i
-  return -1
-
-
-# arraycopy(Object src, int srcPos, Object dest, int destPos, int length)
-# Copies an array from the specified source array, beginning at the specified position, to the specified position of the destination array.
-def System_arraycopy(src, srcPos, dest, destPos, length):
-  for i in range(0, length):
-    dest[destPos+i] = src[srcPos+i]
-
-def getDirectory(path):
-  '''
-   * Return directory path of a given path.
-   * @param path
-   * @return
-  '''
-  if path:
-    idx = path.rfind(os.sep)
-    return path[0:idx] if idx>0 else ''  
-
-  return None
+# def Arrays_binarySearch(t, key):
+#   ''' Locate the leftmost value exactly equal to x '''
+#   i = bisect_left(t, key)
+#   if i != len(t) and t[i] == key:
+#     return i
+#   return -1
 
+# def getDirectory(path): --> use os.path.dirname
 
 def RemoveExt(filename):
   posext = -1 if (filename == None) else filename.rfind('.')
-  return filename[0:posext] if (posext > 0) else filename
+  return filename[0:posext] if posext>0 else filename
 
 
 def cmp_to_key(mycmp):
@@ -107,7 +83,4 @@ def cmp_to_key(mycmp):
             return mycmp(self.obj, other.obj) != 0
     return K
 
-if __name__ == '__main__':
-  print(GetLowExt('toto.AFG'))
-  
   
