@@ -4,17 +4,17 @@ import os
 import sys
 import logging
 from collections import namedtuple
+from bisect import bisect_left
 
-from langutil import C_OK, C_FAIL, C_ERROR, Arrays_binarySearch, C_UNKNOWN
+from langutil import C_OK, C_FAIL, C_ERROR, C_UNKNOWN
 
 from pypos3d.wftk.PoserFileParser import PoserFileParser
 
 from pypos3d.pftk.PoserBasic import cleanName, getRealPath, index, PoserConst, PoserToken, Lang, create, WBoolLine
 from pypos3d.pftk.SimpleAttribut import LinkParmsSA
-from pypos3d.pftk.StructuredAttribut import GenericTransform, ValueOpDelta, Deltas, ChannelMorphStatus, ChannelMorphStatusList, PoserMaterial, Keys, calcMapping_KDTree, findNewDelta, filterLength
+from pypos3d.pftk.StructuredAttribut import GenericTransform, ValueOpDelta, Deltas, ChannelMorphStatus, ChannelMorphStatusList, PoserMaterial, Keys, calcMapping_KDTree, findNewDelta, filterLength, CustomData
 from pypos3d.pftk.GeomCustom import GeomCustom
-from pypos3d.pftk.PoserMeshed import PoserProp, PoserActor, BaseProp,\
-  PoserMeshedObject
+from pypos3d.pftk.PoserMeshed import PoserProp, PoserActor, BaseProp, PoserMeshedObject
 from pypos3d.pftk.ChannelImportAnalysis import ChannelImportAnalysis
 
 
@@ -69,6 +69,8 @@ class Figure():
     self._attribute = None    
 
     self._figureResFile = frf
+    
+    self._customData = None # Contain a Dictionnary of 'key' : custData()
 
     # Channels for computing actions
     self._lstCS = None
@@ -490,6 +492,28 @@ class Figure():
     
     return C_OK
   
+  
+  
+  # Customer Data Management
+  def setCustomData(self, key, val, withPose=0, withMat=0):
+    if not self._customData:
+      self._customData = CustomData()
+      
+    self._customData.set(key, val, withPose, withMat)
+
+  def getCustomData(self, key=None):
+    ''' Return figure's custom data.
+    if key is null : return the underlying dictionnary key:custData(val, withPose, withMath)
+    if key is not null : return the stored cusData
+    '''
+    if not self._customData:
+      return None
+      
+    return self._customData.get(key) if key else self._customData._data
+
+  def hasCustomData(self):
+    return self._customData!=None
+  
   #
   # Options of the report algorithm
   # class ReportOption(object): Named Tuple from PoserMeshed
@@ -1031,7 +1055,9 @@ class Figure():
           dlt = gt.getDeltas()
           for dp in dlt.getDeltaTab():
             wholeMapIdx = tmpmap[dp.getPointNo()]
-            pos = Arrays_binarySearch(nVertIdx, wholeMapIdx)
+            # pos = Arrays_binarySearch(nVertIdx, wholeMapIdx)
+            pi = bisect_left(nVertIdx, wholeMapIdx)
+            pos = pi if pi != len(nVertIdx) and nVertIdx[pi] == wholeMapIdx else -1
             if pos < 0:
               logging.warning("Morphs remapping error for " + pa.getGeomGroupName() + "." + gt.getName())
               ret = C_ERROR
@@ -1389,7 +1415,9 @@ class Figure():
     replace = False
     sonName, parentName = '', ''
     
-    for k,v in dico.items():
+    # FIX: Python 3.8 does not support dictionnary change during iteration
+    # for k,v in dico.items():
+    for (k,v) in [ (k,v) for k,v in dico.items() ]:
       if oldPartName==v.son:
         del dico[k]
         sonName = newPartName
@@ -1556,6 +1584,8 @@ class Figure():
       self._inkyChains.append(po)
     elif po.getPoserType()==PoserToken.E_material:
       self._lstMat.append(po)
+    elif po.getPoserType()==PoserToken.E_customData:
+      self._customData = po
     else:
       logging.info("Figure[%s] - Illegal structured:%s", self._printName, po.getPoserType().token)
       
@@ -1577,9 +1607,6 @@ class Figure():
     for pa in self._lstActor:
       pa.writeData(fw, "")
 
-    #for pp in self._lstProp:
-    #  pp.writeData(fw, "")
-
     # Write FigureDescription part
     fw.write(pfx + 'figure\n')
     nextPfx = pfx + "\t"
@@ -1619,6 +1646,10 @@ class Figure():
     if self._attribute:
       fw.write(nextPfx + "attribute " + self._attribute + '\n')
       
+    # Write Custom Data if any  
+    if self._customData:
+      self._customData.write(fw, nextPfx)
+      
     fw.write(nextPfx + "}\n")
 
 
