@@ -976,9 +976,9 @@ class PoserToken:
   E_stiffExpo = PToken("stiffExpo", False, None, False)
   E_keepLengths = PToken("keepLengths", False, None, False)
   E_weightVertPct = PToken("weightVertPct", False, None, False)
-  E_customData = PToken("customData", True, "StructuredAttribut", False)
+  E_customData = PToken("customData", True, "CustomData", False)
   # PToken.E_frontImage = PoserToken("frontImage", True, FrontImage.__class__)
-  E_data = PToken("data", False, None, False)
+  # Not used Anymore E_data = PToken("data", False, None, False)
   E_weightMap = PToken("weightMap", True, "WeightMap", False) # Poser9
   E_Zones = PToken("zones", True, "Zones", False) # Poser9
   E_WeightMapZone = PToken("weightmapzone", True, "WeightMapZone", False) # Poser9
