@@ -9,7 +9,7 @@ from pypos3d.wftk.PoserFileParser import PoserFileParser, ParsingErrorException
 from pypos3d.pftk.PoserBasic import PoserConst, PoserToken, PToken, buildRelPath, getRealPath, index, TAS, WBoolLine, create, Lang
 from pypos3d.pftk.GeomCustom import GeomCustom
 from pypos3d.pftk.StructuredAttribut import StructuredAttribut, AlternateGeom, Deltas, ValueOpDelta, ChannelMorphStatus, \
-    GenericTransform, ChannelMorphStatusList, findNewDelta, filterLength, calcMapping_KDTree
+    GenericTransform, ChannelMorphStatusList, findNewDelta, filterLength, calcMapping_KDTree, CustomData
 
 
 #
@@ -89,6 +89,8 @@ class PoserMeshedObject(StructuredAttribut):
     self._locked = False
     self._lstAltGeom = [ ]  # For Python, never null
     self._lstWMap = [ ] # List of Weight Maps (could be implemented with a ordered dict)
+
+    self._customData = None # Contain a Dictionnary of 'key' : custData()
   
     #  Name + Mesh definition
     self._printName = None
@@ -489,6 +491,26 @@ class PoserMeshedObject(StructuredAttribut):
 
   def setOrientation(self, v):
     self._orientation = v
+    
+  # Customer Data Management
+  def setCustomData(self, key, val, withPose=0, withMat=0):
+    if not self._customData:
+      self._customData = CustomData()
+      
+    self._customData.set(key, val, withPose, withMat)
+
+  def getCustomData(self, key=None):
+    ''' Return figure's custom data.
+    if key is null : return the underlying dictionnary key:custData(val, withPose, withMath)
+    if key is not null : return the stored cusData
+    '''
+    if not self._customData:
+      return None
+      
+    return self._customData.get(key) if key else self._customData._data
+
+  def hasCustomData(self):
+    return self._customData!=None
 
   # Create in the current MeshedObject the best deltas for the list of morph
   # targets.
@@ -816,6 +838,10 @@ class PoserMeshedObject(StructuredAttribut):
     elif po.getPoserType()==PoserToken.E_readScript:
       #  TODO : Read Channel Definition Script
       logging.error("No Management rule for %s [%s]", po.getName(), po.getPoserType().token)
+      
+    elif po.getPoserType()==PoserToken.E_customData:
+      self._customData = po
+
     else:
       super(PoserMeshedObject, self).addAttribut(po)
 
@@ -963,6 +989,11 @@ class PoserMeshedObject(StructuredAttribut):
       po.write(fw, nextPfx)
     if len(self._lstAttr) == 0:
       fw.write('\n')
+      
+    # Write Customer Data
+    if self._customData:
+      self._customData.write(fw, nextPfx)
+
     fw.write(nextPfx + "}\n")
 
   def write(self, fw, pfx):
