@@ -1633,6 +1633,49 @@ class SphereZone(StructuredAttribut):
   def __init__(self, Name=None):
     super(SphereZone, self).__init__()
 
+# E_Data Tuple Key/Value for Customer Data Storage
+custData = namedtuple('custData', ['val', 'withPose', 'withMat'])
+
+class CustomData(StructuredAttribut):
+  ''' Represents the customData structure that can be found at figure/actor/prop level '''
+  def __init__(self, Name=None):
+    super(CustomData, self).__init__(Name)
+    self._data = {}
+
+  def set(self, key, val, withPose=0, withMat=0):
+    self._data[key] = custData(val, withPose, withMat)
+
+  def get(self, key):
+    return self._data.get(key, None)
+
+  # Overloaded reader    
+  def read(self, st):
+    while True:
+      code,cn,rw = st.getLine()
+      if code== PoserFileParser.TT_WORD:
+        if cn=="data":
+          tv = rw.split()
+          self.set(tv[0], RemoveQuotes(tv[3]), int(tv[1]), int(tv[2]))
+        else: #  Mot illegal
+          logging.info("Line[%s] - Not accepted word:%s",st.lineno(), cn)
+      elif (code==PoserFileParser.TT_RIGHTBRACKET) or (code==PoserFileParser.TT_EOF): 
+        break
+      else:
+        logging.warning("Line[%s] - Not Accepted :%s", st.lineno(), cn)
+        raise ParsingErrorException()
+    # End of read
+
+  def write(self, fw, pfx):
+    fw.write(pfx + self.getPoserType().token + " " + self.getName() + '\n')
+    nextPfx = pfx + "\t"
+    fw.write(nextPfx + "{\n")
+
+    nextPfx = pfx + '\tdata {0:s} {1:d} {2:d} "{3:s}"\n'
+
+    for k,d in self._data.items():
+      fw.write(nextPfx.format(k, d.withPose, d.withMat, d.val))
+ 
+    fw.write(pfx + "\t}\n")
 
 
 # 
