@@ -372,7 +372,7 @@ class MxStdModel(object):
           for i,v in enumerate(vertIdxTbl):
             mapVT[v] = tvertIdxTbl[i]
       except IndexError:
-        logging.warn("Geom[%s] has bad texture index", geom.getName())
+        logging.warning("Geom[%s] has bad texture index", geom.getName())
         hasTexture = False
         
       # Write out the texture coordinates
