@@ -567,19 +567,19 @@ class GeomGroup(object):
     # Verify Vertex indexes
     for vi in self.vertIdx:
       if (vi<0) or (vi>=sizeVertexList):
-        print('Group[{0:s} Bad Vertex Index: {1:d}'.format(name, vi))
+        print('Group[{0:s}] Bad Vertex Index: {1:d}'.format(name, vi))
         res = C_ERROR
     
     if self.tvertIdx:
       if len(self.tvertIdx)!=len(self.vertIdx):
-        print('Group[{0:s} Bad Length for Texture Index: {1:d} instead of {2:d}'.format(name, len(self.tvertIdx),len(self.vertIdx)))
+        print('Group[{0:s}] Bad Length for Texture Index: {1:d} instead of {2:d}'.format(name, len(self.tvertIdx),len(self.vertIdx)))
         res = C_ERROR
 
       sizeTexList = len(self.geom.texList)
         
       for vti in self.tvertIdx:
         if (vti<0) or (vti>=sizeTexList):
-          print('Group[{0:s} Bad Texture Index List: {1:d}'.format(name, vti))
+          print('Group[{0:s}] Bad Texture Index List: {1:d}'.format(name, vti))
           res = C_ERROR
         
     if self.normIdx:
