@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 # package: pypos3d.wftk
 import sys
+import os
 import copy
 import logging
 import traceback
@@ -10,7 +11,7 @@ import bisect
 import numpy as np
 from scipy import spatial
 
-from langutil import C_OK, C_FAIL, C_ERROR, System_arraycopy
+from langutil import C_OK, C_FAIL, C_ERROR
 from pypos3d.wftk import WFBasic 
 from pypos3d.wftk.WFBasic import C_MISSING_FACEMAT, C_MISSING_MAT, FEPSILON, CoordSyst, FaceNormalOrder
 from pypos3d.wftk.WFBasic import LowestIdxPos, findCommonPoints, CreateLoop, FaceVisibility
@@ -21,6 +22,68 @@ from pypos3d.wftk.Repere import Repere
 from pypos3d.wftk.GeomGroup import GeomGroup
 
 
+class WFMat:
+  ''' Simple WaveFront Material.
+  Only diffuse color, diffuse map and transparency are managed.
+  '''
+  
+  def __init__(self, name, kd=(0.9, 0.9, 0.9), map_kd=None):
+    self.name = name
+    self.d = 1.0 # Opacity
+    self.kd = kd if len(kd)==4 else kd + (self.d, )
+    self.map_kd = map_kd
+    # self.ka = (0.0, 0.0, 0.0)
+    # self.ks = (0.0, 0.0, 0.0)
+    # self.ke = (0.0, 0.0, 0.0)
+    # self.illum = 2.0
+    
+
+  @classmethod
+  def readMtl(cls, libMaterialName):
+    logging.info("Loading Material file:%s", libMaterialName)
+
+    rin = open(libMaterialName, 'rt', errors='replace') 
+
+    parser = PoserFileParser(rin)
+    matDict = {}
+    curMat = None
+    
+    while True:
+      code,lw,rw = parser.getLine()
+      lw = lw.lower()
+      
+      if code==PoserFileParser.TT_WORD:
+        if lw=="newmtl": # New material
+          curMat = WFMat(rw)
+          matDict[rw] = curMat
+
+        elif (lw=="ka") or (lw=="ks") or (lw=='ke') or (lw=='illum'):
+          #tv = rw.split()
+          #curMat.ka = ( float(tv[0]), float(tv[1]), float(tv[2]), float(tv[3]) )
+          pass
+        
+        elif lw=="kd":
+          tv = rw.split()
+          curMat.kd = (float(tv[0]), float(tv[1]), float(tv[2]), curMat.d)
+
+        elif lw=="d":
+          curMat.d = float(rw)
+          curMat.kd = (curMat.kd[0], curMat.kd[1], curMat.kd[2], curMat.d)
+
+        elif lw=="map_kd":
+          curMat.map_kd = rw
+      elif code==PoserFileParser.TT_EOF:
+        break
+      else:
+        raise Exception("L[%d] - Not Accepted : %s", parser.lineno(), lw)
+
+    rin.close()
+
+    return matDict
+
+  
+
+
 # =====================================================================================
 class WaveGeom(object):
   '''WaveGeom class represent a WaveFront file and a geometric container for 3D objects
@@ -112,25 +175,50 @@ class WaveGeom(object):
 
     # Name of file
     self._libMaterialName = ''
-    
+    self.libMat = {}
 
-  def __init__(self, *arg, **kwargs):
+  def __init__(self, st=None, filename=None, usemtl=False, imgdirpath='.'):
     ''' Create a Wavefront geometry.
     Initialize temporaly lists for reading operations.
     Parameters
     ----------
     st : optional input file 
-      if None, Create a first default internal group named "default".
+      if None, Create an empty WaveGeom with a first default internal group 
+      named "default".
+      
+    usemtl : optional, bool, default is False
+      Indicate if material files (*.mtl) shall be read
+      
+    imgdirpath : optional, str, default is None
+      list of directories separated by os.pathsep where to search
+      texture files. (for usemtl = True)
+      
     '''
     self._selfInit()
-
+    self.setName(filename)
+    
     self.curGroup = GeomGroup("default")
     self.curGroup.linkCoordList(self)
+    self.usemtl = False
+    self.imgdirpath = imgdirpath
 
-    try:
-      st = kwargs['st']
+    if st:
+      self.usemtl = usemtl
+      
       self.readInternal(st)
-    except KeyError:
+      
+      if self.usemtl and self._libMaterialName:
+        # Load the given .mtl File
+        try:
+          filedir = os.path.dirname(filename)
+          
+          self.libMat = WFMat.readMtl(os.path.join(filedir, self._libMaterialName))
+          
+          # TODO: Check Material usage in groups
+        except FileNotFoundError:
+          logging.warning("Material file read Error:%s", self._libMaterialName)
+          
+    else:
       # When the WaveGeom is not created from a file 
       # a default material is required
       self.addMaterial("default") 
@@ -144,6 +232,7 @@ class WaveGeom(object):
   def getName(self): return self._name
   def getGroups(self): return self.groups
   def getMaterialList(self): return self.lstMat
+  def getMaterialLibName(self): return self._libMaterialName  
 
   def setTexList(self, ntextab):
     self.texList = ntextab
@@ -317,6 +406,21 @@ class WaveGeom(object):
     return self.curMatIdx
 
 
+  def sanityCheck(self):
+    ''' Check the consistency of a WaveGeom. '''
+    
+    gret = C_OK
+    
+    for grp in self.groups:
+      print('Checking Group[{:s}]:'.format(grp.getName()))
+      ret = grp.sanityCheck()
+      if ret!=C_OK:
+        print('Group has errors')
+        gret = min(gret, ret)
+        
+    return gret
+
+
   # TODO Accelerate with KDTree and PaveList2D
   def optimizeGroups(self, cleaning=False):
     ''' Optimize the groups of this WaveGeom.
@@ -653,6 +757,36 @@ class WaveGeom(object):
   def hasNormals(self):
     return self.normList != None and len(self.normList)
 
+
+
+# TODO: Compute Normals if none and required
+#     if self.hasNormal and not geom.normList:
+#       nidx = 0 
+#       for grp in geom.getGroups():
+#         vertIdxTbl = grp.vertIdx
+#         
+#         for noface in range(0, grp.getNbFace()):
+#           startIdx = grp.getFaceStartIdx(noface)
+#           lastIdx = grp.getFaceLastIdx(noface)
+#           argc = lastIdx - startIdx
+#   
+#           v0 = vertIdxTbl[startIdx]
+#           v1 = vertIdxTbl[startIdx+1]
+#           v2 = vertIdxTbl[startIdx+2]
+# 
+#           lv1 = geom.coordList[v0]     
+#           a = Point3d(geom.coordList[v1]).sub(lv1)
+#           b = Point3d(geom.coordList[v2]).sub(lv1)
+#           n = a.cross(b)
+#           n.normalize()
+#           
+#           geom.normList.append(n)
+#           
+#           # Give the same normal to each vertex (Ugly)
+#           grp.normIdx.extend( [nidx] * argc )
+# 
+#           nidx += 1
+
   # public void writeVertex(PrintWriter fw, String nPfx, boolean writeNormals)
   def writeVertex(self, fw, nPfx, writeNormals): # PrintWriter fw, String nPfx, boolean writeNormals)
     # Print Vertex
@@ -815,8 +949,6 @@ class WaveGeom(object):
 
     return ret
 
-  # public String getMaterialLibName() { return _libMaterialName } 
-
   # Find the Egde vextex based on 'findEdges' method.
   # Group is supposed to be alone in the GeomCustom.
   #   
@@ -1188,7 +1320,7 @@ class WaveGeom(object):
     for curGeom in inLst:
       # Create a mapping table for Vertex of that GeomCustom
       mapVert = [ ]
-      prevMax = len(tmpCoordList)
+      #prevMax = len(tmpCoordList)
 
       # Record the mapping table for external usage
       if outMapLst:
@@ -1595,7 +1727,8 @@ class WaveGeom(object):
     ngrp = self.createGeomGroup(destGrpName)
 
     ncl = repUVN.reserveProject(nbEdgePt, self.getCoordListLength(), nbNewPt, edgeSamp)
-    System_arraycopy(self.getCoordList(), 0, ncl, 0, self.getCoordListLength())
+    #System_arraycopy(self.getCoordList(), 0, ncl, 0, self.getCoordListLength())
+    ncl[0:self.getCoordListLength()] = self.getCoordList()[0:self.getCoordListLength()]
     self.setCoordList(ncl)
     if destMatName:
       self.addMaterial(destMatName)
@@ -1656,7 +1789,7 @@ class WaveGeom(object):
 
 
 # ------------------------------------------------------------------------------
-def readGeom(fn):
+def readGeom(fn, usemtl=False, imgdirpath=''):
   ''' Read a .OBJ (or .OBZ compressed) file and return a WaveGeom.
   
   Parameters
@@ -1676,8 +1809,7 @@ def readGeom(fn):
     rin = getOBJFile(fn)
 
     pfr = PoserFileParser(rin, OBJ_FNAT)
-    gm = WaveGeom(st=pfr)
-    gm.setName(fn)
+    gm = WaveGeom(st=pfr, filename=fn, usemtl=usemtl, imgdirpath=imgdirpath)
 
     rin.close()
 
@@ -2066,7 +2198,7 @@ def PlaneSplit(target, centerOrRepOrPlane, eu=None, ev=None, radialLimit=0.0, ma
   #Plot(closingFaceVx,None)
   
   if materialName:    
-    lstFacesIdx, nbFaces, cl = topGrp.addFaceByEdges(closingFaceVx, hasTexture, materialName, refNorm=Vector3d(cuttingVect).neg())
+    lstFacesIdx, nbFaces, _ = topGrp.addFaceByEdges(closingFaceVx, hasTexture, materialName, refNorm=Vector3d(cuttingVect).neg())
   else:
     lstFacesIdx, nbFaces  = None, 0
 
@@ -2210,7 +2342,7 @@ def PlaneSlice(target, centerOrRepOrPlane, eu=None, ev=None, cutFaceMatLst=None,
     lstEdgeVx = [ (vxtab[i], vxtab[i+1]) for i in range(0, len(vxtab)-1) ] + [ (vxtab[-1], vxtab[0]), ]
 
     # Now the plan is at the origin - The Normal is enough to cut
-    lstNewFaceVx, lstNewEdges, multiple = FaceCut(lstEdgeVx, cuttingVect, hasTexture, True, radialLimit2)
+    _, lstNewEdges, _ = FaceCut(lstEdgeVx, cuttingVect, hasTexture, True, radialLimit2)
  
     # Add the new Edges to the closing Face
     if lstNewEdges:
@@ -2219,7 +2351,7 @@ def PlaneSlice(target, centerOrRepOrPlane, eu=None, ev=None, cutFaceMatLst=None,
         cutFaceMatLst.append(target.matIdx[faceno])
   
   # Compute the closing face(s) and keep the first one
-  lstEdgesList, loccl, s  = CreateLoop(closingFaceVx)
+  lstEdgesList, loccl, _  = CreateLoop(closingFaceVx)
   lstEdges = lstEdgesList[0] if lstEdgesList else [ ]
 
   # Scale if required 
@@ -2415,7 +2547,7 @@ def RadialScaleRemesh(target, centerOrRepOrPlane, eu=None, ev=None, dh=0.0, ds=0
     repTop = topPlane.calcCoordSyst() if topPlane else CoordSyst(Point3d(center).add(Vector3d(0.0, 0.0, dh).inLin33(repBottom.MT)), repBottom.eu, repBottom.ev)
 
   # Slice the objet in three groups (bottom group is tmpCd.ogrp)
-  tmpWg, tmpCd = PlaneSplit(target, repBottom, materialName='bottomTmpFace', radialLimit=radialLimit)
+  _, tmpCd = PlaneSplit(target, repBottom, materialName='bottomTmpFace', radialLimit=radialLimit)
 
   if tmpCd.nbFaces!=1:
     logging.warning('  ({0:s})-Bottom Extremity slice does not contain one face but:{1:d}'.format(target.getName(), tmpCd.nbFaces))
